package OAIParser;
#
# Parse XML OAI v1.0 responses. 
# Simeon Warner - 1May2001...
#
# [CVS: $Id: OAIParser.pm,v 1.2 2001/06/06 13:43:24 simeon Exp $]
use strict;

use Carp qw(carp croak verbose);
use XML::Parser;

my $GUNZIP="gunzip --to-stdout"; #command to gunzip to stdout
my $debug=0;

sub new {
  my ($class, %args) = @_;
  $args{Style}='Tree';
  my $parser=new XML::Parser(%args);
  $args{parser}=$parser;
  bless \%args, $class;
}

sub parseFile {
  my $self = shift;
  my $file = shift;
  local(*FILE);
  if ($file=~/\.(Z|gz)$/) {
    open(FILE, "$GUNZIP $file |") or croak "Couldn't open $file:\n$!";
  } else {
    open(FILE, $file) or croak "Couldn't open $file:\n$!";
  }
  binmode(FILE);
  my @ret = $self->parse(*FILE, @_);
  close(FILE);
  return(@ret);
}

sub parse {
  my $self = shift;
  my $tree;
  eval {
    $tree = $self->{parser}->parse(@_);
  };
  my $err = $@;
  if ($err) {
    croak "Error from Expat: $err\n";
  } else {
    $self->{tree}=$tree;
  }
  return($err);
}


# Returns
#  resumptionToken (string) on success
#  undef, undef             if no resumptionToken found
#  undef, error message     if some other error
#
sub getResumptionToken {
  my $self = shift;
  my $resumptionToken = $self->find(['*','resumptionToken'],1);
  return(undef,undef) unless defined($resumptionToken);
  my $rt2 = $self->find(['*','resumptionToken'],2);
  return(undef,'duplicate resumptionToken') if defined($rt2);
  return($resumptionToken);
}

# Returns
#  responseDate, datestamp  on success
#  undef, undef             if no resumptionToken found
#  undef, error message     if some other error
#
sub getResponseDate {
  my $self = shift;
  my $responseDate = $self->find(['*','responseDate'],1);
  return(undef,undef) unless defined($responseDate);
  my $rd2 = $self->find(['*','responseDate'],2);
  return(undef,"duplicate responseDate") if defined($rd2);
  # extract datestamp of 'today' from responseDate
  return(undef,"can't extract datestamp from responseDate") unless ($responseDate=~/^(\d{4}\-\d{2}\-\d{2})/);
  return($responseDate,$1);
}


# Parameters:
#   $fields - ptr to array of elements to match, top level first
#   $occurrence - match the specified occurrence (default=1st)
#             (if occurrence<0, just return number of occurrences)  
#
# Accepts '*' for a wildcard element name to match.
# So $fields = [ '*', 'resumptionToken' ] will match any resumptionToken
# element at second level in tree.
#
# Returns:
#   string - if match is a CDATA element
#   ref    - if match is a subtree
#   undef  - if no match
#
sub find {
  my $self = shift;
  my $fields = shift; #pointer to array that gives field we are looking for
  my $occurrence = shift; #occurrence we are looking for
  $occurrence=1 unless ($occurrence); 
  #
  my @stack=();
  unless (defined($self->{tree})) {
    carp "Call to find without parse tree!";
    return(undef);
  }
  my @cc = @{$self->{tree}}; #copy so as not to destroy tree
  my $contents=\@cc;
  my $level = 0; # keep track of level in tree
  my $count = 0; # count matches
  while ($level>-1) {
    while (scalar(@$contents)>0) {
      my $tag = shift(@$contents);
      my $val = shift(@$contents);
      if ($tag eq '0') {
        #content, not element, $val is content
      } else {
        print "find: ".'  ' x $level if ($debug);
        #element $tag, $val is ptr to array
        print "<$tag> " if ($debug);
        if ($tag eq $fields->[$level] or $fields->[$level] eq '*') {
	  if (($level+1) == scalar(@$fields)) {
            my @c=@$val;
            my $atts=shift(@c);
            $count++;
            if ($count==$occurrence) {
              if (scalar(@c)==2 and $c[0] eq '0') { 
       	        print "match element '".$c[1]."'\n" if ($debug);
    	        return($c[1]);
	      } else { 
   	        print "match subtree\n" if ($debug);
                return([$tag,$val]);
	      }
	    } else {
	      print "match $count, want $occurrence\n" if ($debug);
	    }
	  } else {
            # got match, descend into tree
            print "match, descending\n" if ($debug);
  	    push(@stack, $contents);
            my @cv=@$val; #copy so as not to destroy tree
            $contents=\@cv;
            my $atts=shift(@$contents);
            $level++;
	  }
        } else {
	  print "no match, not descending\n" if ($debug);
        }
      }
    }
    # Run to end of contents at level $level, try to go up
    $contents=pop(@stack);
    $level--;
  }
  return($count) if ($occurrence<0); 
  return(undef);
}

# print out tree (for debugging)
#
sub printTree {
  my $self = shift;
  my ($subtree,$reformat,$indent)=@_;
  my $tree=(defined($subtree)?$subtree:$self->{tree});
  if (defined($indent)) { $indent.=' '; } else { $indent=''; }
  my ($element,$contents)=@$tree;
  my @c=@$contents; #copy so we don't destroy real tree
  my $args = shift @c;
  print "<$element";
  if (scalar(%$args)>0) {
    foreach my $k (keys %$args) {
      print " $k=".$args->{$k};
    }
  }
  print ">";
  my $singleContent=(scalar(@c)==2);
  while (scalar(@c)>0) {
    my $tag = shift @c;
    my $val = shift @c;
    if ($tag eq '0') {
      unless (!$singleContent and $val=~/^\s+$/ and $reformat) {
        print $val;
      } else { 
        print "\n$indent";
      }
    } else {
      print ' ' if ($reformat);
      $self->printTree([$tag,$val],$reformat,$indent);
    }
  }
  print "</$element>";
}

1;

